'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ATTRIBUTES_TO_PROPS = exports.ATTRIBUTES = exports.FILTER_CAST_BOOL = exports.FILTER_CAST_NUMBER = exports.FILTER_DENY = exports.FILTER_ALLOW = exports.TAGS_BLACKLIST = exports.TAGS = exports.CONFIG_BLOCK = exports.CONFIG_INLINE = exports.TYPE_INLINE_BLOCK = exports.TYPE_BLOCK = exports.TYPE_INLINE = exports.PARSER_DENY = exports.PARSER_ALLOW = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PARSER_ALLOW = exports.PARSER_ALLOW = 1; /**
                                              * @copyright   2016, Miles Johnson
                                              * @license     https://opensource.org/licenses/MIT
                                              * 
                                              */

var PARSER_DENY = exports.PARSER_DENY = 2;var TYPE_INLINE = exports.TYPE_INLINE = 'inline';
var TYPE_BLOCK = exports.TYPE_BLOCK = 'block';
var TYPE_INLINE_BLOCK = exports.TYPE_INLINE_BLOCK = 'inline-block';var CONFIG_INLINE = exports.CONFIG_INLINE = {
  rule: PARSER_ALLOW,
  type: TYPE_INLINE,
  inline: true,
  block: false,
  self: false,
  void: false,
  parent: [],
  children: []
};

var CONFIG_BLOCK = exports.CONFIG_BLOCK = {
  rule: PARSER_ALLOW,
  type: TYPE_BLOCK,
  inline: true,
  block: true,
  self: true,
  void: false,
  parent: [],
  children: []
};

var tagConfigs = {
  a: {
    type: TYPE_INLINE_BLOCK,
    block: true
  },
  address: {
    self: false
  },
  audio: {
    children: ['track', 'source']
  },
  br: {
    inline: false,
    void: true
  },
  button: {
    type: TYPE_INLINE_BLOCK
  },
  dd: {
    parent: ['dl']
  },
  dl: {
    children: ['dt']
  },
  dt: {
    parent: ['dl'],
    children: ['dd']
  },
  figcaption: {
    parent: ['figure']
  },
  footer: {
    self: false
  },
  header: {
    self: false
  },
  h1: {
    self: false
  },
  h2: {
    self: false
  },
  h3: {
    self: false
  },
  h4: {
    self: false
  },
  h5: {
    self: false
  },
  h6: {
    self: false
  },
  hr: {
    inline: false,
    block: false,
    void: true
  },
  img: {
    inline: false,
    void: true
  },
  li: {
    self: false,
    parent: ['ul', 'ol']
  },
  main: {
    self: false
  },
  ol: {
    children: ['li']
  },
  picture: {
    children: ['source', 'img']
  },
  source: {
    inline: false,
    parent: ['audio', 'video', 'picture'],
    void: true
  },
  span: {
    self: true
  },
  table: {
    children: ['thead', 'tbody', 'tfoot', 'tr']
  },
  tbody: {
    parent: ['table'],
    children: ['tr']
  },
  td: {
    parent: ['tr']
  },
  tfoot: {
    parent: ['table'],
    children: ['tr']
  },
  th: {
    parent: ['tr']
  },
  thead: {
    parent: ['table'],
    children: ['tr']
  },
  tr: {
    parent: ['table', 'tbody', 'thead', 'tfoot'],
    children: ['th', 'td']
  },
  track: {
    inline: false,
    parent: ['audio', 'video'],
    void: true
  },
  ul: {
    children: ['li']
  },
  video: {
    children: ['track', 'source']
  }
};

function createConfigBuilder(config) {
  return function (tagName) {
    tagConfigs[tagName] = (0, _extends3.default)({}, config, tagConfigs[tagName]);
  };
}

['a', 'abbr', 'b', 'br', 'button', 'cite', 'code', 'del', 'dfn', 'em', 'i', 'img', 'ins', 'kbd', 'label', 'mark', 'output', 'picture', 'q', 's', 'samp', 'source', 'span', 'strong', 'sub', 'sup', 'time', 'track', 'u', 'var', 'video'].forEach(createConfigBuilder(CONFIG_INLINE));

['address', 'article', 'aside', 'audio', 'blockquote', 'dd', 'details', 'div', 'dl', 'dt', 'fieldset', 'figcaption', 'figure', 'footer', 'header', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'legend', 'li', 'main', 'nav', 'ol', 'p', 'pre', 'section', 'summary', 'table', 'thead', 'tbody', 'tfoot', 'tr', 'th', 'td', 'ul'].forEach(createConfigBuilder(CONFIG_BLOCK));

var TAGS = exports.TAGS = Object.freeze(tagConfigs);

var TAGS_BLACKLIST = exports.TAGS_BLACKLIST = {
  applet: true,
  body: true,
  canvas: true,
  embed: true,
  frame: true,
  frameset: true,
  head: true,
  html: true,
  iframe: true,
  object: true,
  script: true,
  style: true
};

var FILTER_ALLOW = exports.FILTER_ALLOW = 1;
var FILTER_DENY = exports.FILTER_DENY = 2;
var FILTER_CAST_NUMBER = exports.FILTER_CAST_NUMBER = 3;
var FILTER_CAST_BOOL = exports.FILTER_CAST_BOOL = 4;

var ATTRIBUTES = exports.ATTRIBUTES = Object.freeze({
  alt: FILTER_ALLOW,
  cite: FILTER_ALLOW,
  class: FILTER_ALLOW,
  colspan: FILTER_CAST_NUMBER,
  controls: FILTER_CAST_BOOL,
  datetime: FILTER_ALLOW,
  default: FILTER_CAST_BOOL,
  disabled: FILTER_CAST_BOOL,
  dir: FILTER_ALLOW,
  height: FILTER_ALLOW,
  href: FILTER_ALLOW,
  id: FILTER_ALLOW,
  kind: FILTER_ALLOW,
  label: FILTER_ALLOW,
  lang: FILTER_ALLOW,
  loop: FILTER_CAST_BOOL,
  muted: FILTER_CAST_BOOL,
  poster: FILTER_ALLOW,
  role: FILTER_ALLOW,
  rowspan: FILTER_CAST_NUMBER,
  span: FILTER_CAST_NUMBER,
  src: FILTER_ALLOW,
  target: FILTER_ALLOW,
  title: FILTER_ALLOW,
  width: FILTER_ALLOW
});

var ATTRIBUTES_TO_PROPS = exports.ATTRIBUTES_TO_PROPS = Object.freeze({
  class: 'className',
  colspan: 'colSpan',
  datetime: 'dateTime',
  rowspan: 'rowSpan'
});